"""
OSX implementation of usb kernel driver detachment.

As kextunload and kextload requires root privilege,
one can use this module with sudo or change the /etc/sudoers file accordingly:

your_login ALL=(ALL) NOPASSWD: /sbin/kextload, /sbin/kextunload

"""

import subprocess
from dn.usb.common import wait_for_driver
import logging

FTDI_BUNDLE_ID = 'com.FTDI.driver.FTDIUSBSerialDriver'
APPLE_FTDI_BUNDLE_ID = 'com.apple.driver.AppleUSBFTDI'

KEXT_FIND = '/usr/sbin/kextfind'
KEXT_UNLOAD = '/sbin/kextunload'
KEXT_LOAD = '/sbin/kextload'


def find_bundle(bundle_name):
    """find driver."""
    found = subprocess.check_output([KEXT_FIND, '-b', bundle_name])
    return len(found) > 0


CURRENT_BUNDLE_ID = (
    FTDI_BUNDLE_ID if find_bundle(FTDI_BUNDLE_ID)
    else (
        APPLE_FTDI_BUNDLE_ID if find_bundle(APPLE_FTDI_BUNDLE_ID)
        else None
        )
    )


def detach_kernel_driver(serial):
    """detach kernel driver."""
    logging.debug('{} is selected as the corresponding kernel driver'.format(
        CURRENT_BUNDLE_ID))

    if is_kernel_driver_active(serial):
        if subprocess.call(["sudo",
                            KEXT_UNLOAD,
                            "-b",
                            CURRENT_BUNDLE_ID]) != 0:
            logging.error(
                'Kernel driver not properly detached, proceeding anyways')
            # TODO: Find better way to proceed when kernel cannot be detached.
        else:
            logging.info('Kernel driver detached')
    else:
        logging.warning('Kernel driver already detached')


def is_kernel_driver_active(_):
    """check if kernel driver is attached."""
    loaded = subprocess.check_output([
        KEXT_FIND, '-loaded', '-b', CURRENT_BUNDLE_ID])
    return len(loaded) > 0


def attach_kernel_driver(path, _):
    """attach kernel driver."""
    logging.debug('Attaching kernel driver')
    if subprocess.call(["sudo", KEXT_LOAD, "-b", CURRENT_BUNDLE_ID]) != 0:
        logging.warning(
            'Kernel driver not properly attached, proceeding anyways')
    wait_for_driver(path)
    logging.info('Kernel driver attached')



#!/usr/bin/env python

"""tc65i main module used from command line."""

from __future__ import print_function
import argparse
import itertools
import functools
import collections
import re
import logging
import os
import dateutil.parser as dateparser
import sys

import serial

import dn.ftdicbus as cbus
from dn.tc65i import obex, at


logger = logging.getLogger(__name__)

TIMEOUT = 3.0
BAUDRATE = 115200

MAX_CLOSE_TRY = 5


class Tc65i(object):

    """Tc65i modem manager."""

    File = collections.namedtuple(
        'File', 'type permission owner group size modified name')
    File.aligns = '<<<<><<'

    def __init__(self, ser):
        """initialize Tc65i object."""
        self.obex_mode = False
        self.ser = ser

    def __enter__(self):
        """called when with statement is executed."""
        return self

    def __exit__(self, _, value, traceback):
        self.close_obex()

    def open_obex(self):
        """switch into obex data mode."""
        if not self.obex_mode:
            at.send(self.ser, 'AT\\Q3')
            at.send(self.ser, 'AT^SQWE=0')
            at.send(self.ser, 'AT^SQWE=3')
            obex.connect(self.ser)
            self.obex_mode = True
            logger.info('obex mode opened')

    def close_obex(self):
        """switch into at command mode."""
        if self.obex_mode:
            obex.disconnect(self.ser)
            for close_try in xrange(MAX_CLOSE_TRY):
                try:
                    logger.debug(
                        'trying (%d left) to close obex mode' % close_try)
                    at.send(self.ser, '+++', term_char='', timeout=1)
                    break
                except at.ATException:
                    pass
            else:
                raise IOError(
                    'Cannot close obex mode. tc65i does not '
                    'respond to +++ after %d tries' % MAX_CLOSE_TRY)
            at.send(self.ser, 'ATE')
            self.obex_mode = False
            logger.info('obex mode closed')

    def obexer(func):
        def actual(*args, **kwargs):
            """perform obex guarding and call the wrapping function."""
            self = args[0]
            if not self.obex_mode:
                self.open_obex()
            args = (self.ser,) + args[1:]
            return getattr(obex, func.__name__)(*args, **kwargs)
        return functools.wraps(func)(actual)

    def ater(func):
        def actual(*args, **kwargs):
            """perform at guarding and call the wrapping function."""
            self = args[0]
            if self.obex_mode:
                self.close_obex()
            args = (self.ser,) + args[1:]
            return getattr(at, func.__name__)(*args, **kwargs)
        return functools.wraps(func)(actual)

    @ater
    def send(*args, **kwargs):
        pass

    @ater
    def getconfig(*args, **kwargs):
        pass

    @obexer
    def getfile(*args, **kwargs):
        pass

    @obexer
    def putfile(*args, **kwargs):
        pass

    @obexer
    def listdir(*args, **kwargs):
        pass

    @obexer
    def changedir(*args, **kwargs):
        pass

    @obexer
    def removeobject(*args, **kwargs):
        pass

    @obexer
    def erasestorage(*args, **kwargs):
        pass

    @obexer
    def makedir(*args, **kwargs):
        pass

    @obexer
    def proctriplet(*args, **kwargs):
        pass

    @obexer
    def abort(*args, **kwargs):
        pass

    def freespace(self):
        """retrieve the free space."""
        return self.proctriplet(obex.Triplet.frombytes([0x32, 0x01, 0x02]))

    def totalspace(self):
        """retrieve the total space."""
        return self.proctriplet(obex.Triplet.frombytes([0x32, 0x01, 0x01]))

    def lsconfig(self):
        """return formatted configuration lines."""
        res = []
        conf_ptrn = re.compile('("[^\"]*")')
        for line in self.getconfig():
            confstr = conf_ptrn.findall(line)
            if confstr:
                res.append('AT^SCFG={},{}'.format(
                    confstr[0], ','.join(confstr[1:])))
        return '\n'.join(res)

    def lsall(self, parent=''):
        def check_perm(perm, ctx):
            return perm if perm in ctx else '-'

        def ls_perms(ctx):
            """format permission line."""
            return check_perm(
                'R', ctx) + check_perm('W', ctx) + check_perm('D', ctx)

        rootelem = self.listdir()
        if len(rootelem) == 0:
            return []

        files = []
        for child in rootelem:
            uperm = child.attrib.get('user-perm', '')
            gperm = child.attrib.get('group-perm', '')
            operm = child.attrib.get('other-perm', '')
            mod = child.attrib.get('modified', '')
            if mod:
                mod = dateparser.parse(mod)
            perm = ('d' if child.tag == 'folder' else '-') + ls_perms(uperm)
            perm += ls_perms(gperm) + ls_perms(operm)
            name = child.attrib.get('name', '')
            file_ = Tc65i.File(
                child.attrib.get('type', ''),
                perm,
                child.attrib.get('owner', ''),
                child.attrib.get('group', ''),
                child.attrib.get('size', '0'),
                mod,
                parent+'/' + name,
                )
            files.append(file_)
            if file_.permission[0] == 'd' and 'R' in file_.permission:
                self.changedir(dir_=name)
                kids = self.lsall(parent=(file_.name))
                self.changedir("..")
                files += kids
        return files

    @classmethod
    def formatfiles(cls, files):
        """format file information."""
        def max_item_len(items, ind):
            return max([len(str(item[ind])) for item in items])

        titles = [str.capitalize(f) for f in cls.File._fields]

        files_str = [
            tuple([str(v) for v in f._asdict().values()]) for f in files]

        max_lengths = [
            max_item_len([titles] + files_str, i) for i in xrange(len(titles))]

        titlef_str = ' '.join(
            ['{{:^{}}}'] * len(max_lengths)).format(*max_lengths)

        f_str = ' '.join([
            '{{:{}{}}}'] * len(max_lengths)).format(*itertools.chain(*zip(
                cls.File.aligns, max_lengths)))
        file_list = [f_str.format(*f) for f in files_str]
        return '\n'.join([titlef_str.format(*titles)] + file_list)


class Run(argparse.Action):

    """argparse helper."""

    def __init__(self, option_strings, dest, **kwargs):
        super(Run, self).__init__(option_strings, dest, **kwargs)
        self.func = getattr(kwargs, 'func', None)

    def __call__(self, parser, namespace, values, option_string=None):
        if not hasattr(namespace, 'actions'):
            namespace.actions = []
        if len(values) == 0:
            pairs = [(self, None)]
        elif isinstance(values, basestring):
            pairs = [(self, values)]
        else:
            pairs = [(self, v) for v in values]
        namespace.actions.extend(pairs)


def cb_print_settings(modem, path, args):
    """print modem settings."""
    print(modem.lsconfig())


def cb_list_dir(modem, path, args):
    """list directory contents."""
    files = modem.lsall()
    print('Available {} of {} bytes'.format(
        modem.freespace(), modem.totalspace()))
    print(modem.formatfiles(files))


def cb_put_file(modem, path, args):
    """put file (to remote --destination directory optionally)."""
    destpath = args.destination + '/' + os.path.basename(path)
    with open(path, 'r') as fp_:
        modem.removeobject(destpath)
        modem.putfile(destpath, fp_)
    logger.info('transferred {} to {}'.format(path, destpath))


def cb_get_file(modem, path, args):
    """get file (to local --destination directory optionally)."""
    destfile = sys.stdout
    if args.destination:
        destfile = open(args.destination, 'w')
    try:
        destfile.write(modem.getfile(path))
    finally:
        if args.destination:
            destfile.close()


def cb_make_dir(modem, path, args):
    """create new directory."""
    logger.info('creating ' + path)
    modem.makedir(path)


def cb_remove(modem, path, args):
    """delete file or directory."""
    logger.info('deleting ' + path)
    modem.removeobject(path)


def cb_format(modem, path, args):
    """erase entire storage."""
    logger.info('formatting ...')
    modem.erasestorage()
    logger.info('formatted')


def cb_run(modem, path, args):
    """run jar on the modem."""
    logger.info('running {} ...'.format(path))
    match = re.match(r'(a\:)?\/*(.*)', path)
    res = modem.send('AT^SJRA="a:/{}"'.format(match.group(2)))
    logger.info('AT result: {}'.format(res))


def cb_abort(modem, path, args):
    """abort running application."""
    logger.info('aborting ...')
    modem.abort()


def cb_atcmd(modem, path, args):
    """execute AT command and print the result."""
    logger.info('running at command {} ...'.format(path))
    res = modem.send(path)
    logger.info(res)


def cb_dump(modem, path, args):
    """dump from modem."""
    try:
        while True:
            sys.stdout.write(modem.ser.read(1))
            sys.stdout.flush()
    except KeyboardInterrupt:
        pass


def main():
    """call module from command line."""
    root_logger = logging.getLogger('')
    handler = logging.StreamHandler()
    root_logger.addHandler(handler)
    root_logger.setLevel(logging.ERROR)

    parser = argparse.ArgumentParser()
    act_grp = parser.add_argument_group(
        'available commands (all are recursive)')
    act_grp.add_argument(
        '-ls', '--list-dir', help=cb_list_dir.__doc__,
        action=Run, nargs=0).func = cb_list_dir
    act_grp.add_argument(
        '-pf', '--put-file', metavar='FILE', nargs='+',
        help=cb_put_file.__doc__, action=Run).func = cb_put_file
    act_grp.add_argument(
        '-gf', '--get-file', metavar='PATH', nargs='+',
        help=cb_get_file.__doc__, action=Run).func = cb_get_file
    act_grp.add_argument(
        '-rm', '--remove', metavar='PATH', nargs='+',
        help=cb_remove.__doc__, action=Run).func = cb_remove
    act_grp.add_argument(
        '-md', '--make-dir', metavar='PATH', nargs='+',
        help=cb_make_dir.__doc__, action=Run).func = cb_make_dir
    act_grp.add_argument(
        '-fm', '--format', action=Run, nargs=0,
        help=cb_format.__doc__).func = cb_format
    act_grp.add_argument(
        '-ru', '--run', action=Run, metavar='PATH', nargs=1,
        help=cb_run.__doc__).func = cb_run
    act_grp.add_argument(
        '-ab', '--abort', action=Run, nargs=0,
        help=cb_abort.__doc__).func = cb_abort
    act_grp.add_argument(
        '-at', '--exec-at', action=Run, metavar='COMMAND', nargs='+',
        help=cb_atcmd.__doc__).func = cb_atcmd
    act_grp.add_argument(
        '-ps', '--print-settings', help=cb_print_settings.__doc__,
        action=Run, nargs=0).func = cb_print_settings
    act_grp.add_argument(
        '-du', '--dump', help=cb_dump.__doc__,
        action=Run, nargs=0).func = cb_dump

    dbg_grp = parser.add_mutually_exclusive_group()
    dbg_grp.add_argument(
        '-d', '--debug', action='store_true',
        help='print debugging information')
    dbg_grp.add_argument(
        '-v', '--verbose', action='store_true',
        help='try to be a bit more verbose')

    parser.add_argument(
        '-n', '--no-cbus', action='store_true',
        help='do not change cbus pins')
    parser.add_argument(
        '-D', '--destination', metavar='PATH',
        help='destination directory', default='')
    parser.add_argument(
        'port', metavar='PATH',
        help='serial device to connect')

    args = parser.parse_args()

    if not hasattr(args, 'actions'):
        print('No commands defined')
        parser.print_help()
        return

    if args.verbose:
        root_logger.setLevel(logging.INFO)
    if args.debug:
        handler.setFormatter(logging.Formatter(
            '[%(filename)s:%(lineno)d] %(message)s'))
        root_logger.setLevel(logging.DEBUG)

    if not args.no_cbus:
        cbus.set_cbus_pins(args.port, cbus.TC65I_MODE)
    try:
        with serial.serial_for_url(args.port) as ser:
            ser.timeout = TIMEOUT
            ser.baudrate = BAUDRATE
            with Tc65i(ser) as modem:
                if hasattr(args, 'actions'):
                    for action, param in args.actions:
                        action.func(modem, param, args)
    finally:
        if not args.no_cbus:
            cbus.set_cbus_pins(args.port, cbus.MSP430_UART_MODE)


if __name__ == '__main__':
    main()



"""handle at command transmission and receiving the response."""

import logging

logger = logging.getLogger(__name__)

E_OK = 'OK'
E_ERROR = 'ERROR'
LE = '\r\n'

AT_CONFIG = 'AT^SCFG'
AT_Q = '?'


class ATException(Exception):

    """raised when AT command related exception occurs."""

    pass


def send(serdev, cmd, term_char=LE, timeout=None):
    logger.debug('Sending {} to {}'.format(cmd, serdev.name))
    if timeout:
        old = serdev.timeout
        serdev.timeout = timeout
    try:
        serdev.flushInput()
        serdev.flushOutput()
        serdev.write(cmd + term_char)
        serdev.flush()
        response = []
        while True:
            line = serdev.readline()
            logger.debug('AT reply: {}'.format(line))
            if not line:
                raise ATException(
                    response if response
                    else (
                        'No reply from %s within %.1f sec, make sure tc65i'
                        ' is powered, attached, or not running MIDlet'
                        % (serdev.name, serdev.timeout))
                    )
            elif line == LE:
                continue
            elif line == E_OK + LE:
                success = True
                break
            elif line == E_ERROR + LE:
                success = False
                break
            else:
                response.append(line.strip())
        logger.debug('Result: ({}) {}'.format(success, response))
        return success, response
    finally:
        if timeout:
            serdev.timeout = old


def getconfig(serdev, ret_success=False):
    """read configuration of the modem."""
    res = send(serdev, AT_CONFIG + AT_Q)
    return res if ret_success else res[1]



import sys as _sys

if _sys.platform.startswith("darwin"):
    from dn.usb.macosx import is_kernel_driver_active, detach_kernel_driver
    from dn.usb.macosx import attach_kernel_driver
elif _sys.platform.startswith('linux'):
    from dn.usb.linux import is_kernel_driver_active, detach_kernel_driver
    from dn.usb.linux import attach_kernel_driver
else:
    raise Exception("Sorry. Only mac and linux are supported")



"""linux implementation of usb kernel driver detachment."""

import logging
from dn.usb.common import wait_for_driver, usb_find_by_serial

logger = logging.getLogger(__name__)


def detach_kernel_driver(serial):
    """detach kernel driver for given serial."""
    dev = usb_find_by_serial(serial)
    if dev.is_kernel_driver_active(0):
        logger.debug('Detaching kernel driver')
        dev.detach_kernel_driver(0)
    if dev.is_kernel_driver_active(0):
        logger.warning(
            'Kernel driver not properly attached. Proceeding anyways')
    logger.info('Kernel driver detached')


def is_kernel_driver_active(serial):
    """check kernel driver attached for given serial."""
    dev = usb_find_by_serial(serial)
    return dev.is_kernel_driver_active(0)


def attach_kernel_driver(path, serial):
    """attach kernel driver for given serial."""
    dev = usb_find_by_serial(serial)
    if not dev.is_kernel_driver_active(0):
        dev.attach_kernel_driver(0)
        wait_for_driver(path)
        logger.info('Kernel driver attached')



#!/usr/bin/env python
"""
Sets FTDI CBUS bits using the bit-banging mode.

CBUS is used to select active target connected
FTDI serial interface. Following modes are defined:
    MSP430_UART_MODE    = 0x30
    MSP430_BSL_MODE     = 0x31
    TC65I_MODE          = 0x32
"""

import argparse
import ftdi
import re
import logging
import contextlib

from dn import usb

logger = logging.getLogger(__name__)

MSP430_UART_MODE = 0x30
MSP430_BSL_MODE = 0x31
TC65I_MODE = 0x32

FTDI_VENDOR = 0x0403
FTDI_PRODUCT = 0x6001


def extract_serial(device):
    """return serial number from device path."""
    match = re.match('^.*tty.usbserial-(.*)$', device)
    if match:
        serial = match.group(1)
        return serial
    logger.warn("Serial number is not recognized %s", device)


@contextlib.contextmanager
def open_ftdi(device):
    """open before and to close after the operation."""
    serial = extract_serial(device)
    usb.detach_kernel_driver(serial)
    try:
        ftdic = ftdi.ftdi_new()
        if not ftdic:
            raise IOError("Cannot allocate ftdi context")
        try:
            if serial is None:
                usb_desc = ftdi.ftdi_usb_open(
                    ftdic, FTDI_VENDOR, FTDI_PRODUCT)
            else:
                usb_desc = ftdi.ftdi_usb_open_desc(
                    ftdic, FTDI_VENDOR, FTDI_PRODUCT,
                    None, serial)
            if usb_desc < 0:
                raise IOError(usb_desc, "Can't open ftdi device")
            try:
                yield ftdic
            finally:
                ftdi.ftdi_usb_close(ftdic)
        finally:
            ftdi.ftdi_free(ftdic)
    finally:
        usb.attach_kernel_driver(device, serial)


def set_cbus_pins(device, mode, ftdic=None):
    """set cbus pins in CBUS mode."""
    with open_ftdi(device) as ftdic:
        ftdi.ftdi_set_bitmode(ftdic, mode, ftdi.BITMODE_CBUS)
        logger.info("ftdi cbus pins changed to: {0:#x}".format(mode))


def get_cbus_pins(device, ftdic=None):
    """read value of cbus pins."""
    with open_ftdi(device) as ftdic:
        pins = ftdi.new_ucharp()
        val = ftdi.ftdi_read_pins(ftdic, pins)
        res = ftdi.ucharp_value(val)
        logger.debug("ftdi cbus pins read: {0:#x}".format(res))


def main():
    """use module functions from command line."""
    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group(required=True)
    choices = {
        'msp430-uart': MSP430_UART_MODE,
        'msp430-bsl': MSP430_BSL_MODE, 'tc65i': TC65I_MODE}
    group.add_argument(
        '-m', '--mode', metavar='MODE', choices=choices.keys(),
        help='set predefined mode (%s)' % (', '.join(choices.keys())))
    group.add_argument(
        '-s', '--set', metavar='VALUE', help='set raw value in 0xHH format')
    group.add_argument(
        '-r', '--read', action='store_true',
        help='print raw value in 0xHH format')
    parser.add_argument(
        '-p', '--port', metavar='PATH',
        help='serial device path', required=True)
    parser.add_argument(
        '-v', '--verbose', action='store_true', help='verbose output')
    parser.add_argument(
        '-d', '--debug', action='store_true', help='debug output')
    parser.add_argument(
        '-t', '--attach-timeout', metavar='SECONDS',
        help='''timeout until device node appears after attaching kernel
                driver (default is %.1f sec)''' % usb.common.WAIT_TIMEOUT)
    args = parser.parse_args()

    root_logger = logging.getLogger('')
    handler = logging.StreamHandler()
    root_logger.addHandler(handler)
    root_logger.setLevel(logging.ERROR)

    if args.verbose:
        root_logger.setLevel(logging.INFO)

    if args.debug:
        root_logger.setLevel(logging.DEBUG)
        handler.setFormatter(logging.Formatter(
            '[%(filename)s:%(lineno)d] %(message)s'))

    if args.attach_timeout:
        usb.common.WAIT_TIMEOUT = float(args.attach_timeout)

    if args.mode:
        set_cbus_pins(args.port, choices[args.mode])
    elif args.set:
        set_cbus_pins(args.port, int(args.set, 16))
    elif args.read:
        print get_cbus_pins(args.port)


if __name__ == '__main__':
    main()



"""build and install the python modules and command line scripts."""
from setuptools import setup, find_packages
import os


def read(*paths):
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join(*paths), 'r') as fp_:
        return fp_.read()

setup(
    name='decent-tools',
    version='0.1.10',
    description='decentnode tools',
    long_description=(read('README.rst')),
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Topic :: System :: Systems Administration',
        ],
    url='http://www.decentlab.com',
    author='hashiname',
    author_email='hashiname@gmail.com',
    license='Apache License 2.0',
    packages=find_packages(exclude='tests'),
    entry_points={
        'console_scripts': [
            'dn-bsl = dn.bsl:main',
            'dn-ftdi-cbus = dn.ftdicbus:main',
            'dn-tc65i = dn.tc65i.__main__:main',
            'dn-motes = dn.usb.common:print_motes'
            ],
        },
    install_requires=[
        'pyserial>=2.5, <3.0',
        'python-msp430-tools>=0.7, <1.0',
        'python-dateutil>=2.0, <3.0',
        ],
    include_package_data=True,
    test_suite='nose.collector',
    tests_require=['nose'],
    zip_safe=False,
    dependency_links=[
        'bzr+lp:python-msp430-tools/trunk/@python-msp430-tools-0.7'
        '#egg=python_msp430_tools-0.7'],
    )



#!/usr/bin/env python
"""decentnode bsl programmer."""

import logging
import time
import sys
import argparse

import msp430
import msp430.bsl.target as bsl
from msp430 import memory

from dn import ftdicbus


class DnBsl(bsl.SerialBSLTarget, object):
    """Class providing direct access to the BSL of decentnode."""
    def __init__(self):
        """initialize the object."""
        self.download_data = None
        self.options = None
        self.total = 0
        self.current = 0
        super(DnBsl, self).__init__()

    def reset(self):
        """Reset the device."""
        logging.info('Resetting device')

        self.set_RST(True)
        self.set_TEST(True)
        time.sleep(0.25)

        self.set_RST(False)
        self.set_RST(True)

        time.sleep(0.25)
        self.serial.flushInput()

    def add_dn_options(self, args):
        """add decentnode specific options."""
        self.options, _ = self.parser.parse_args([])
        self.options.invert_test = True
        self.options.invert_reset = True
        self.options.input_format = 'ihex'
        self.options.speed = 38400
        self.options.port = args.port

    def add_program(self, args):
        self.add_dn_options(args)
        if not args.no_erase:
            self.options.do_mass_erase = True
            self.add_action(self.mass_erase)

        self.options.do_program = True
        self.add_action(self.program_file)

        self.download_data = memory.Memory()
        data = memory.load(args.firmware, format=self.options.input_format)
        self.download_data.merge(data)
        self.add_action(self.reset)

        segs = [
            len(seg) + 1 if len(seg) & 1 else len(seg)
            for seg in self.download_data]
        self.total = sum(segs)
        self.current = 0

    def add_reset(self, args):
        self.add_dn_options(args)
        self.options.do_reset = True
        self.add_action(self.reset)

    def open_connection(self):
        """FIXME: remove this override ASAP. Find better way."""
        self.logger = logging.getLogger('BSL')
        self.open(
            self.options.port,
            ignore_answer=self.options.ignore_answer,
        )
        self.control_delay = self.options.control_delay

        if self.options.extra_erase_cycles is not None:
            self.main_erase_cycles += self.options.extra_erase_cycles

        if self.options.test_on_tx:
            self.testOnTX = True

        if self.options.invert_test:
            self.invertTEST = True

        if self.options.invert_reset:
            self.invertRST = True

        if self.options.swap_reset_test:
            self.swapResetTest = True

        self.set_TEST(True)
        self.set_RST(True)

        if self.options.do_reset:
            return

        if self.options.start_pattern:
            self.start_bsl(self.options.prompt_before_release)

        if self.options.do_mass_erase:
            self.extra_timeout = 6
            self.mass_erase()
            self.extra_timeout = None
            self.BSL_TXPWORD('\xff'*32)
            # remove mass_erase from action list so that it is not done
            # twice
            self.remove_action(self.mass_erase) 
        else:
            if self.options.password is not None:
                password = msp430.memory.load(self.options.password).get_range(0xffe0, 0xffff)
                self.logger.info("Transmitting password: %s" % (password.encode('hex'),))
                self.BSL_TXPWORD(password)

        # check for extended features (e.g. >64kB support)
        self.logger.debug('Checking if device has extended features')

        if not self.options.do_mass_erase:
            self.BSL_TXPWORD('\xff'*32)

        self.check_extended()

        if self.options.replace_bsl:
            family = msp430.target.identify_device(self.device_id, self.bsl_version)
            if family == msp430.target.F1x:
                bsl_name = 'BL_150S_14x.txt'
            elif family == msp430.target.F4x:
                bsl_name = 'BL_150S_44x.txt'
            else:
                raise bsl.BSLError('No replacement BSL for %s' % (family,))

            self.logger.info('Download replacement BSL as requested by --replace-bsl')
            replacement_bsl_txt = pkgutil.get_data('msp430.bsl', bsl_name)
            replacement_bsl = msp430.memory.load('BSL', StringIO(replacement_bsl_txt), format='titext')
            self.program_file(replacement_bsl)

            bsl_start_address = struct.unpack("<H", replacement_bsl.get(0x0220, 2))[0]
            self.execute(bsl_start_address)
            self.logger.info("Starting new BSL at 0x%04x" % (bsl_start_address,))
            time.sleep(0.050)   # give BSL some time to initialize
            #~ if self.options.password is not None:
                #~ self.BSL_TXPWORD(password)
        else:
            if self.bsl_version <= 0x0110:
                self.logger.info('Buggy BSL, applying patch')
                patch_txt = pkgutil.get_data('msp430.bsl', 'patch.txt')
                patch = msp430.memory.load('PATCH', StringIO(patch_txt), format='titext')
                self.program_file(patch)
                self.patch_in_use = True

        if self.options.speed is not None:
            try:
                self.set_baudrate(self.options.speed)
            except bsl.BSLError:
                raise bsl.BSLError("--speed option not supported by BSL on target")

    def memory_write(self, address, data):
        """write to flash memory."""
        if len(data) & 1:  # padding
            data += '\xff'
        while data:
            block, data = data[:self.MAXSIZE], data[self.MAXSIZE:]
            if self.extended_address_mode:
                self.BSL_SETMEMOFFSET(address >> 16)
            self.BSL_TXBLK(address & 0xffff, bytes(block))
            address += len(block)
            self.current += len(block)
            if self.verbose:
                sys.stderr.write("\r Transferred {} of {} bytes ".format(
                    self.current, self.total))


def main():
    """run from command line."""
    dnbsl = DnBsl()

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-nc', '--no-cbus', action='store_true',
        help='do not change cbus pins')
    dbg_grp = parser.add_mutually_exclusive_group()
    dbg_grp.add_argument(
        '-d', '--debug', action='store_true',
        help='print debugging information')
    dbg_grp.add_argument(
        '-v', '--verbose', action='store_true',
        help='try to be a bit more verbose')

    subs = parser.add_subparsers(title='commands')

    reset_p = subs.add_parser('reset', help='Reset mote')
    reset_p.set_defaults(func=dnbsl.add_reset)

    program_p = subs.add_parser('program', help='Interactive shell')
    program_p.add_argument(
        '-f', '--firmware', metavar='PATH',
        help='binary firmware to transfer', required=True)
    program_p.add_argument(
        '-ne', '--no-erase', action='store_true',
        help='do not send mass erase command before programming')
    program_p.set_defaults(func=dnbsl.add_program)
    parser.add_argument(
        'port', metavar='PATH', help='serial port to connect')
    args = parser.parse_args()

    root_logger = logging.getLogger('')
    handler = logging.StreamHandler()
    root_logger.addHandler(handler)
    root_logger.setLevel(logging.ERROR)
    dnbsl.verbose = 0

    if args.verbose:
        dnbsl.verbose = 1
        root_logger.setLevel(logging.INFO)

    if args.debug:
        dnbsl.verbose = 3
        root_logger.setLevel(logging.DEBUG)
        handler.setFormatter(logging.Formatter(
            '[%(filename)s:%(lineno)d] %(message)s'))

    if not args.no_cbus:
        ftdicbus.set_cbus_pins(args.port, ftdicbus.MSP430_BSL_MODE)
    try:
        dnbsl.create_option_parser()
        dnbsl.add_extra_options()
        args.func(args)
        dnbsl.do_the_work()
    finally:
        if not args.no_cbus:
            ftdicbus.set_cbus_pins(args.port, ftdicbus.MSP430_UART_MODE)


if __name__ == '__main__':
    main()



"""Helper module for kernel driver attaching."""
from __future__ import print_function
import os
import time
import logging
import usb

WAIT_TIMEOUT = 5
FTDI_VENDOR = 0x0403
FTDI_PRODUCT = 0x6001
SERIAL_MAX_LEN = 256


def wait_for_driver(device_path):
    """wait until the device file is present."""
    elapsed = 0
    logging.debug('waiting for driver')
    while True:
        if os.path.exists(device_path):
            try:
                tryee = os.open(device_path, os.O_NONBLOCK)
                if tryee > 0:
                    os.close(tryee)
                    return
            except OSError:
                pass
        time.sleep(0.01)
        elapsed = elapsed + 0.01
        if elapsed > WAIT_TIMEOUT:
            raise IOError('Not accessible: ' + str(device_path))


def test_wait_with_manual_plug():
    """immediately plug ftdi once running this function."""
    wait_for_driver('/dev/tty.usbserial-A900gwLV')
    return True


def usb_find_by_serial(serial):
    """return usb device with given serial."""
    devs = usb.core.find(
        find_all=True, idVendor=FTDI_VENDOR, idProduct=FTDI_PRODUCT)
    for dev in devs:
        if usb.util.get_string(dev, SERIAL_MAX_LEN,
                               dev.iSerialNumber) == serial:
            return dev
    raise IOError("No ftdi mote is found")


def print_motes():
    """return usb devices with given serial."""
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', action='store_true', help='Compact format')
    args = parser.parse_args()
    found = usb.core.find(
        find_all=True, idVendor=FTDI_VENDOR, idProduct=FTDI_PRODUCT)
    serials = [
        usb.util.get_string(dev, SERIAL_MAX_LEN, dev.iSerialNumber)
        for dev in found]
    paths = ['/dev/tty.usbserial-' + s for s in serials]
    if not args.c:
        path_title = 'Expected path'
        serial_title = 'Serial'
        serial_max = max([len(s) for s in serials + [serial_title]])
        path_max = max([len(s) for s in paths + [path_title]])
        print(
            '%s %s' %
            (serial_title.center(serial_max), path_title.center(path_max)))
        print('%s %s' % ('-' * serial_max, '-' * path_max))
    for s, p in zip(serials, paths):
        if args.c:
            print('%s,%s' % (s, p))
        else:
            print('%s %s' % (s.ljust(serial_max), p.ljust(path_max)))
        



"""implements basic obex protocol module."""

import array
import struct
import logging
import os
import time
import xml.etree.ElementTree as ET

HEADER_COUNT = 0xC0
HEADER_NAME = 0x01
HEADER_TYPE = 0x42
HEADER_LENGTH = 0xC3
HEADER_TIME = 0x44
HEADER_DESCRIPTION = 0x05
HEADER_TARGET = 0x46
HEADER_HTTP = 0x47
HEADER_BODY = 0x48
HEADER_END_OF_BODY = 0x49
HEADER_WHO = 0x4A
HEADER_CONNECTION_ID = 0xCB
HEADER_APP_PARAMETERS = 0x4C
HEADER_AUTH_CHALLENGE = 0x4D
HEADER_AUTH_RESPONSE = 0x4E
HEADER_CREATOR_ID = 0xCF
HEADER_WAN_UUID = 0x50
HEADER_OBJECT_CLASS = 0x51
HEADER_SESSION_PARAMETERS = 0x52
HEADER_SESSION_SEQUENCE_NUMBER = 0x93

REQUEST_CONNECT = 0x80
REQUEST_DISCONNECT = 0x81
REQUEST_PUT = 0x02
REQUEST_GET = 0x03
REQUEST_SETPATH = 0x85
REQUEST_SETPATH2 = 0x86
REQUEST_SESSION = 0x87
REQUEST_ABORT = 0xFF
REQUEST_FINAL = 0x80

FLAG_SETPATH_CREATE = 0x00
FLAG_SETPATH_NOCREATE = 0x02
FLAG_SETPATH_PARENT_FOLDER = 0x03

RESPONSE_SUCCESS = 0x20
RESPONSE_CONTINUE = 0x10
RESPONSE_CREATED = 0x21
RESPONSE_BADREQUEST = 0x40
RESPONSE_FINAL = 0x80

MAX_PART_LEN = 512

logger = logging.getLogger(__name__)


class ObexException(Exception):
    pass


class Triplet(object):

    """Helper class to store triple object."""

    def __init__(self, tag, len_, value):
        self.tag = tag
        self.len_ = len_
        self.value = value

    @classmethod
    def frombytes(cls, bytes_):
        return cls(bytes_[0], bytes_[1], bytes_[2:])

    def bytes(self):
        return [self.tag, self.len_] + self.value


def bytestoint(bytes_, short=0):
    if len(bytes_) > 0:
        return (bytes_[-1] << short) + bytestoint(bytes_[:-1], short+8)
    return 0


def shorttobytes(shrt):
    return strtobytes(struct.pack('!H', shrt))  # unsigned short 2 bytes


def longtobytes(lng):
    return strtobytes(struct.pack('!L', lng))  # unsigned int 4 bytes


def bytestostr(bytes_):
    return array.array('B', bytes_).tostring()


def strtobytes(str_):
    return array.array('B', str_).tolist()


def buildrequest(opcode, data):
    return [opcode] + shorttobytes(3 + len(data)) + data


def buildheader(type_, data):
    return [type_] + shorttobytes(3 + len(data)) + data


def isvalid_responsecode(code):
    return (
        code & RESPONSE_SUCCESS or
        code & RESPONSE_CREATED or
        code & RESPONSE_CONTINUE)


def parsenullstrfunc(data):
    return (
        3 + bytestoint(data[1:3]),
        bytestostr(data[3:3+bytestoint(data[1:3])]))


def parselenfunc(data):
    return (1 + 4, bytestoint(data[1:1+4]))


def getheaderfunc(type_):
    if type_ == HEADER_COUNT or type_ == HEADER_LENGTH:
        return parselenfunc
    return parsenullstrfunc


def getheaderlen(data):
    return getheaderfunc(data[0])(data)[0]


def getheader(data):
    return getheaderfunc(data[0])(data)[1]


def parseheaders(data):
    if len(data) == 0:
        return dict()
    return dict(
        [(data[0], getheader(data))] +
        parseheaders(data[getheaderlen(data):]).items())


def proctriplet(serdev, triplet):
    """send triplet object and return the result."""
    header = buildheader(HEADER_APP_PARAMETERS, triplet.bytes())
    _, hdrs = procrequest(serdev, REQUEST_PUT | REQUEST_FINAL, header)
    res = Triplet.frombytes(hdrs[HEADER_APP_PARAMETERS])
    return bytestoint(strtobytes(res.value))


def writerequest(serdev, request):
    serdev.flushInput()
    serdev.write(bytestostr(request))
    serdev.flush()


def readresponse(serdev, timeout):
    start = time.time()
    code = ''
    while time.time() - start < timeout or timeout is None:
        code = serdev.read(1)
        if len(code) > 0:
            break
        time.sleep(0.001)
    if len(code) == 0:
        raise ObexException(
            'Empty reply, make sure obex mode is configured on the tc65i')
    logger.debug('response header {:#x}'.format(ord(code)))
    len_str = serdev.read(2)
    length = bytestoint(strtobytes(len_str)) - 3  # code and length are 3 bytes
    logger.debug('response length {:#x}'.format(length))
    data = serdev.read(length)
    logger.debug('response: {}'.format([hex(ord(x)) for x in data]))
    return ord(code), length, strtobytes(data)


def connect(serdev):
    """send connect request."""
    fsuid = [
        0x6b, 0x01, 0xcb, 0x31, 0x41, 0x06, 0x11, 0xd4,
        0x9a, 0x77, 0x00, 0x50, 0xda, 0x3f, 0x47, 0x1f]
    header = buildheader(HEADER_TARGET, fsuid)
    info = [0x13, 0x00] + shorttobytes(0xffff)  # version, flags, max packet length
    procrequest(serdev, REQUEST_CONNECT, info + header)


def disconnect(serdev):
    """send disconnect request."""
    procrequest(serdev, REQUEST_DISCONNECT, shorttobytes(3))


def procrequest(serdev, opcode, data, timeout=10):
    """process a request with opcode and data."""
    request = buildrequest(opcode, data)
    logger.debug('sending {} bytes: {}'.format(
        len(request), [hex(x) for x in request]))

    writerequest(serdev, request)
    code, length, data = readresponse(serdev, timeout=timeout)
    logger.debug("{:#x} len={}: {}".format(code, length, data))

    if not isvalid_responsecode(code):
        raise ObexException('Unknown response code {:#x}'.format(code))
    if code & RESPONSE_BADREQUEST:
        raise ValueError(
            'Bad request received opcode: {:#x} data: {!s}'.format(
                opcode, data))

    headers = parseheaders(data)

    return code, headers


def procbodyrequest(serdev, opcode, data, timeout=10):
    """process body type request with opcode and data."""
    def reqiterator():
        """send and receive responses until RESPONSE_FINAL received."""
        code, headers = procrequest(serdev, opcode, data, timeout=timeout)
        yield headers
        while code & RESPONSE_CONTINUE:
            code, headers = procrequest(
                serdev, REQUEST_GET | REQUEST_FINAL,
                shorttobytes(3), timeout=timeout)
            yield headers

    body = ''
    for headers in reqiterator():
        try:
            body += headers[HEADER_BODY]
        except KeyError:
            logger.info(
                'Body header not found for request 0x%x,'
                ' possibly the last request' % opcode)

    return body


def getfile(serdev, path):
    """retrieve file from the modem."""
    parson = path.split('/', 1)
    if len(parson) > 1:  # go further inside
        changedir(serdev, parson[0])
        res = getfile(serdev, parson[1])
        changedir(serdev, '..')
    elif len(parson) == 1:  # time to get it
        destname = parson[0]
        namehdr = buildheader(
            HEADER_NAME, strtobytes(destname.encode('utf-16be')))
        logger.debug('namehdr: {}'.format([hex(x) for x in namehdr]))
        res = procbodyrequest(serdev, REQUEST_GET | REQUEST_FINAL, namehdr)
    return res


def putfile(serdev, path, file_):
    parson = path.split('/', 1)
    if len(parson) > 1:  # # go further inside
        changedir(serdev, parson[0])
        putfile(serdev, parson[1], file_)
        changedir(serdev, '..')
    elif len(parson) == 1:  # time to put it
        destname = parson[0]
        stat = os.fstat(file_.fileno())
        logger.info('Putting file({} bytes) {} with chunk size of {}'.format(
            stat.st_size, destname, MAX_PART_LEN))
        first = True
        for chunk in iter(lambda: file_.read(MAX_PART_LEN), ''):
            logger.debug('Putting file chunk')
            last = len(chunk) < MAX_PART_LEN
            headers = []
            logger.debug('chunk first:{} last:{}'.format(first, last))
            if first:
                first = False
                namehdr = buildheader(HEADER_NAME, strtobytes(
                    destname.encode('utf-16be')))
                logger.debug('namehdr: {}'.format([hex(x) for x in namehdr]))
                headers.extend(namehdr)
                lenhdr = [HEADER_LENGTH] + longtobytes(stat.st_size)
                logger.debug('lenhdr: {}'.format([hex(x) for x in lenhdr]))
                headers.extend(lenhdr)
                timehdr = buildheader(HEADER_TIME, strtobytes(time.strftime(
                    '%Y%m%dT%H%M%S', time.localtime(stat.st_mtime))))
                logger.debug('timehdr: {}'.format([hex(x) for x in timehdr]))
                headers.extend(timehdr)
            type_ = HEADER_END_OF_BODY if last else HEADER_BODY
            bodyhdr = buildheader(type_, strtobytes(chunk))
            logger.debug('bodyhdr: {}'.format([hex(x) for x in bodyhdr]))
            headers.extend(bodyhdr)
            req = (REQUEST_PUT | REQUEST_FINAL) if last else REQUEST_PUT
            procrequest(serdev, req, headers)


def listdir(serdev):
    """list directory content."""
    header = buildheader(HEADER_TYPE, strtobytes('x-obex/folder-listing'))
    xmlstr = procbodyrequest(serdev, REQUEST_GET | REQUEST_FINAL, header)
    files = ET.fromstring(xmlstr)
    return files


def removeobject(serdev, path):
    """delete file or folder by given path."""
    parson = path.split('/', 1)

    if len(parson) > 1:  # go further inside
        changedir(serdev, parson[0])
        removeobject(serdev, parson[1])  # remove whatever it is
        changedir(serdev, '..')
    elif len(parson) == 1:  # time to remove it
        obj = parson[0]
        folders = listdir(serdev).findall('folder')
        found = [k for k in folders if k.attrib['name'] == obj]
        isfolder = len(found) > 0
        if isfolder:  # then remove kids first
            changedir(serdev, obj)
            for k in listdir(serdev):
                removeobject(serdev, k.attrib['name'])  # remove whatever it is
            changedir(serdev, '..')
        logger.info('Removing object ' + obj)
        namehdr = buildheader(
            HEADER_NAME, strtobytes(parson[0].encode('utf-16be')))
        logger.debug('namehdr: {}'.format([hex(x) for x in namehdr]))
        procrequest(serdev, REQUEST_PUT | REQUEST_FINAL, namehdr)


def makedir(serdev, path):
    """make directory on given path."""
    parson = path.split('/', 1)
    if len(parson) > 0:  # go further inside
        changedir(serdev, parson[0], create=True)
        if len(parson) > 1:
            makedir(serdev, parson[1])
        changedir(serdev, '..')


def changedir(serdev, dir_, create=False):
    if dir_ is None or dir_ == '':
        logger.info('Ignoring invalid directory name \'{}\''.format(dir_))
        return
    elif dir_ == '/' or dir_.lower() == 'a:' or dir_.lower() == 'a:/':
        dir_ = ''

    if dir_ == '..':
        flag = FLAG_SETPATH_PARENT_FOLDER
        header = []
    else:
        flag = FLAG_SETPATH_CREATE if create else FLAG_SETPATH_NOCREATE
        header = buildheader(HEADER_NAME, strtobytes(dir_.encode('utf-16be')))

    logger.info('Changing to {}'.format(dir_))
    procrequest(serdev, REQUEST_SETPATH, [flag, 0x00] + header)


def erasestorage(serdev):
    """format modem storage."""
    erasehdr = buildheader(HEADER_APP_PARAMETERS, [0x31, 0x00])
    logger.debug('erasehdr: {}'.format([hex(x) for x in erasehdr]))
    procrequest(serdev, REQUEST_PUT | REQUEST_FINAL, erasehdr, timeout=100)


def abort(serdev):
    """stop running application on the modem."""
    procrequest(serdev, REQUEST_ABORT | REQUEST_FINAL, [])
